function [solverOutput, methodOptions] = BABGTDE(XCPC, microphonePositions, samplingPeriod, methodOptions, energies)

%Geometric-constrained time difference estimation
%
% USAGE: [solverOutput, methodOptions] = BABGTDE(XCPC, microphones, samplingPeriod, methodOptions, energies)
%
% PARAMETERS:
%   XCPC ~ the interpolation coefficients of the cross-correlation function
%   microphonePositions ~ positions of the microphones
%   samplingPeriod ~ sampling period of the signal
%   methodOptions ~ methodOptions
%   energies ~ the energy of the signals
%
% RESULT:
%   solverOutput ~ the solution of the solver
%   methodOptions ~ the method options eventually modified
%
% DESCRIPTION:
%   Uses a branch and bound algorithm to estimate the time delays.
%

% Copyright 2012, Xavier Alameda-Pineda
% INRIA Grenoble Rhône-Alpes
% E-mail: xavi.alameda@gmail.com
% 
% This is part of the gtde program.
% 
% gtde is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
    %%% Input check
    if nargin < 5
        error('Usage [solverOutput, methodOptions] = BABGTDE(XCPC, microphonePositions, samplingPeriod, methodOptions[, energies])');
    end
    if size(XCPC,1) ~= size(microphonePositions,1)
        error('There should be as many signals as microphones.');
    end
    if size(microphonePositions,2) ~= 3 && size(microphonePositions,2) ~= 2
        error('The system works in 2D or in 3D.');
    end
    if samplingPeriod <= 0
        error('The sampling period should be positive.');
    end
    
    if ~isfield(methodOptions,'init'),
        methodOptions.init = false;
    end
    
    % Declare the objective function
    if nargin < 6
        objFunction = @(x) MultichannelTDECriterion(x,XCPC,microphonePositions,samplingPeriod);
    else
        objFunction = @(x) MultichannelTDECriterion(x,XCPC,microphonePositions,samplingPeriod,energies);
    end        

    % Define the problem
    problem.costFunction = objFunction;
    problem.startingPoint = zeros(size(microphonePositions,1)-1,1);
    problem.startingSize = TDEmax(microphonePositions)/samplingPeriod;
    problem.startingSize = problem.startingSize(1:size(microphonePositions,1)-1);
    problem.constantLipschitz = EstimateLipschitzConstant(microphonePositions,samplingPeriod,objFunction);
    % Set the tolerance of the method
    if isfield(methodOptions,'tolerance')
        problem.tolerance = methodOptions.tolerance;
    else
        problem.tolerance = 0.01;
    end
    problem.maxBranches = 2000;

    % Run B&B
    BaBSolution = BranchAndBound(problem);
    solverOutput.x = BaBSolution.potentialList;
    [flag,solverOutput.f] = objFunction(solverOutput.x);
    solverOutput.c = TDEConstraint(solverOutput.x,microphonePositions,samplingPeriod);
    solverOutput.flag = -flag;
end

% function [Jp, flag] = regularizedgTDECriterion(x,PCCC,microphones,samplingPeriod,energies)
%     % Compute objective function
%     if nargin < 5
%        [Jp, flag] = gTDECriterionBAB(x,PCCC,microphones,samplingPeriod);
%     else
%        [Jp, flag] = gTDECriterionBAB(x,PCCC,microphones,samplingPeriod,energies);
%     end
% 
%     % Compute constraint
%     c = TDEDiscriminant(x,microphones,samplingPeriod);
%     % Add constraint
%     ind = c<0;
%     Jp(ind) = Jp(ind) - 10*c(ind);
% end