function [foundTDE, solverOutput] = EstimateTimeDelays(signals,experimentOptions)

%Warp for all the tested methods. Main function to estimate time delays.
%
% USAGE: [foundTDE, solverOutput] = EstimateTimeDelays(signals,experimentOptions)
%
% PARAMETERS:
%   signals ~ discrete signals received at the microphones
%   experimentOptions ~ options of the experiment
% 
% RETURN VALUE:
%   foundTDE ~ time delay estimates
%   solverOutput ~ output flags of the solver
%
% DESCRIPTION: Estimates the time delays from the discrete signals following
%   the options specified in the structure.
%

% Copyright 2012, Xavier Alameda-Pineda
% INRIA Grenoble Rhône-Alpes
% E-mail: xavi.alameda@gmail.com
% 
% This is part of the gtde program.
% 
% gtde is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
    %%% From signals to Polynomial Coefficient Cross Correlation
    [XCPC, Energies] = Signals2XCPC(signals,experimentOptions.microphonePositions,experimentOptions.samplingFrequency);
    
    % Set the output
    foundTDE = nan(experimentOptions.dimension,1);
    
    % Check if I want to use the constraint or not
    switch experimentOptions.methodOptions.type
        case 'canclini'
            % Using the method in
            %
            % A. Canclini, E. Antonacci, A. Sarti & S. Tubaro. 
            % Acoustic Source Localization With Distributed Asynchronous 
            % Microphone Networks. 
            % Audio, Speech, and Language Processing, IEEE Transactions on
            % vol. 21, no. 2, pages 439–443, 2013.
            [solverOutput, experimentOptions.methodOptions] = CancliniTDE(XCPC,...
                 experimentOptions.microphonePositions,...
                 1./experimentOptions.samplingFrequency,...
                 experimentOptions.initializationPositions,...
                 experimentOptions.methodOptions);
            % Optimal TDE
            foundTDE = solverOutput.x(:,solverOutput.f == min(solverOutput.f));
        case 'bab'
            % Compute TDEs
            [solverOutput, experimentOptions.methodOptions] = BABGTDE(XCPC,...
                 experimentOptions.microphonePositions,...
                 1./experimentOptions.samplingFrequency,...
                 experimentOptions.methodOptions,Energies);
            % Feasible points
            indices = solverOutput.flag >= 0 & solverOutput.c >= 0;
            % Retrieve the optimal point
            if sum(indices)
                foundTDE = solverOutput.x(:,solverOutput.f == min(solverOutput.f(indices)))/experimentOptions.samplingFrequency;
                % If two points are equivalent, we take the first one.
                if size(foundTDE,2) > 1
                    foundTDE = foundTDE(:,1);
                end
            end
        case 'dip'
            % Compute TDEs
            [solverOutput, experimentOptions.methodOptions] = DIPGTDE(XCPC,...
                 experimentOptions.microphonePositions,...
                 1./experimentOptions.samplingFrequency,...
                 (experimentOptions.samplingFrequency*experimentOptions.initializationPositions),...
                 experimentOptions.methodOptions,Energies);
             % Feasible points
            indices = solverOutput.flag' >= 0 & solverOutput.c >= 0;
            % Retrieve the optimal point
            if sum(indices)
                foundTDE = solverOutput.x(:,solverOutput.f == min(solverOutput.f(indices)))/experimentOptions.samplingFrequency;
            end
        case 'ngtde'
            % Compute TDEs
            [solverOutput, experimentOptions.methodOptions] = NGTDE(XCPC,...
                 experimentOptions.microphonePositions,...
                 1./experimentOptions.samplingFrequency,...
                 (experimentOptions.samplingFrequency*experimentOptions.initializationPositions),...
                 experimentOptions.methodOptions,Energies);
            % Feasible points (just the flag, it does not have any sense to
            % ask for the constraint, since the constraint does not exist
            % here
            indices = solverOutput.flag' >= 0;
            % Retrieve the optimal point
            if sum(indices)
                foundTDE = experimentOptions.initializationPositions(:, solverOutput.f == min(solverOutput.f(indices)));
                if sum(indices) >= 2
                    foundTDE = foundTDE(:,1);
                end
            end
                        
        case 'init'
            % Compute the criterion for all the initial values
            [~, foundCriterion] = MultichannelTDECriterion(...
                experimentOptions.samplingFrequency*experimentOptions.initializationPositions,...
                XCPC,...
                experimentOptions.microphonePositions,...
                1/experimentOptions.samplingFrequency,...
                Energies);
            % SolverOutput building
            solverOutput.x = experimentOptions.initializationPositions;
            solverOutput.f = foundCriterion;
            solverOutput.c = TDEConstraint(solverOutput.x,experimentOptions.microphonePositions);
            % Constraints
            indices = solverOutput.c >= 0 & solverOutput.f >= 0;
            % Retrieve optimal point if any
            if sum(indices)
                foundTDE = experimentOptions.initializationPositions(:, solverOutput.f == min(solverOutput.f(indices)));
            end
        case 'bypairs'
            % Compute bypairs
            [solverOutput] = PITDE(XCPC,...
                 experimentOptions.microphonePositions,...
                 1./experimentOptions.samplingFrequency,...
                 experimentOptions.initializationPositions,...
                 experimentOptions.methodOptions);
             % Retrieve optimal poing
             foundTDE = solverOutput.xstar;
        case 'bp2dip'
            experimentOptions.methodOptions.numMaxima = 3;
            % Compute bypairs
            [solverOutput] = PITDE(XCPC,...
                experimentOptions.microphonePositions,...
                1./experimentOptions.samplingFrequency,...
                experimentOptions.initializationPositions,...
                experimentOptions.methodOptions);
            % Get middle optimal points
            middleInitPoints = solverOutput.xstar;
            % Compute the new initialization points
            % That is all the possible combinations of all the values
            % output by the bypairs algorithms
            NPointsDimension = sum(~isnan(middleInitPoints),1);
            NInitPoints = prod(NPointsDimension);
            newInitializationPositions = zeros(experimentOptions.dimension,NInitPoints);
            % Initialize variables
            SubChunkSize = NInitPoints;
            NRepeat = 1;
            % Loop
            for d=1:experimentOptions.dimension,
                % SubChunkSize update
                SubChunkSize = SubChunkSize/NPointsDimension(d);
                Chunk = [];
                for ii=1:NPointsDimension(d),
                    Chunk = cat(2,Chunk,repmat(middleInitPoints(ii,d),1,SubChunkSize));
                end
                newInitializationPositions(d,:) = repmat(Chunk,1,NRepeat);
                % NRepeat update
                NRepeat = NRepeat * NPointsDimension(d);
            end
            % Launch DIP
            [solverOutput, experimentOptions.methodOptions] = DIPGTDE(XCPC,...
                experimentOptions.microphonePositions,...
                1./experimentOptions.samplingFrequency,...
                (experimentOptions.samplingFrequency*newInitializationPositions),...
                experimentOptions.methodOptions,Energies);
            % Feasible points
            indices = solverOutput.flag' >= 0 & solverOutput.c >= 0;
            % Retrieve the optimal point
            if sum(indices)
                foundTDE = solverOutput.x(:,solverOutput.f == min(solverOutput.f(indices)))/experimentOptions.samplingFrequency;
            end
    end % switch
end % EstimateTimeDelays