function [signals, fs] = GetSignalRealization(experimentOptions,sSignal,sPosition,sSNR,t60)

%Get the realization of a signal
%
% [signals, fs] = GetSignalRealization(experimentOptions,sSignal,sPosition,sSNR,sT60)
%
% PARAMETERS:
%   experimentOptions ~ options of the experiment, see batchTDEExperiments
%   sSignal ~ index of the signal
%   sPosition ~ index of the sound source positions
%   sSNR ~ index of the SNR value
%   sT60 ~ index of the t60 value
%
% RETURN VALUE:
%   signals ~ signals received at the microphone(s)
%   fs ~ sampling frequency of the signals
%
% DESCRIPTION:
%   In the case of simulated data, the function will retrieve the correct
%   ISM filter, convolve it with the signal and add some white gaussian
%   noise to the results.
%
%   In the real case, the function will retrieve the corresponding recorded
%   signal.
% 
%   see also batchTDEExperiments

% Copyright 2012, Xavier Alameda-Pineda
% INRIA Grenoble Rhône-Alpes
% E-mail: xavi.alameda@gmail.com
% 
% This is part of the gtde program.
% 
% gtde is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%

% Depending on the data
    % In case of synthetic data, they are functions
    switch experimentOptions.dataOptions.type        
        case 'simulated'
            %%%%%%%%%%%%% WARNING %%%%%%%%%%%%%%%
            % THE NON REVERBERANT CASE IS T60 = 0
            %%%%%%%%%%%%% WARNING %%%%%%%%%%%%%%%
            % Checking the installation of ISM
            if isempty(which('ISM_AudioData'))
                fprintf('The software to compute ISM is not installed.\n');
                fprintf('Please go to http://www.eric-lehmann.com/ism code.html to download and install.\n');
                error('Software missing.');
            end
            % Load the signal
            if sPosition > numel(experimentOptions.signals)
                error('sSignal should be a valid signal index.');
            end
            [signal, fs] = audioread(experimentOptions.signals{sPosition});
            % Sampling frequency index
            sFS = find(experimentOptions.ismOptions.samplingFrequencies == fs);
            if isempty(sFS)
                error(['ISM filters were not computed for sampling frequency of ' num2str(fs) '.']);
            end
            % Sampling frequency index
            sT60 = find(experimentOptions.ismOptions.t60 == t60);
            if isempty(sT60)
                error(['ISM filters were not computed for a T60 of ' num2str(experimentOptions.dataOptions.t60(sT60)) '.']);
            end
            % File name
            ISMFileName = GetISMFileName(experimentOptions,sT60,sPosition,sFS);
            % Check the for the file
            if ~(exist(ISMFileName,'file') == 2)
                error(strcat('ISM filter does not exist (',ISMFileName,').'));
            end
            % Apply RIR filter
            partialSignals = ISM_AudioData(ISMFileName,signal,'SilentFlag',1);
            partialSignals = partialSignals';
            if isfield(experimentOptions.microphonePositionOptions,'subSet'),
                partialSignals = partialSignals(experimentOptions.microphonePositionOptions.subSet,:);
            end
        case 'real'
            % Read the signals
            fileName = strcat(experimentOptions.dataOptions.wavRootFolder,...
                              experimentOptions.signals{sSignal},...
                              '/Recorded/',...
                              num2str(sPosition),'.wav');
            if ~exist(fileName,'file')
                error(['Signal ' fileName ' not found. Problem?']);
            end
            [partialSignals, fs] = audioread(fileName);
            partialSignals = partialSignals';
    end
       
    %%% Cut the signals
    pieceLength = experimentOptions.dataOptions.cutLength*fs; 
    NPieces = floor(size(partialSignals,2)/pieceLength);
    signals = cell(0);

    for np = 1:NPieces,
        auxSignals = partialSignals( :, ((np-1)*pieceLength+1):(np*pieceLength) );
        % Threshold on the input energy
        if sum(sum(auxSignals.^2))/size(experimentOptions.microphonePositions,1) > 5e-3
            if ~strcmp(experimentOptions.dataOptions.type,'real'),
                %%% Add noise
                for channel = 1:size(partialSignals,1),
                    auxSignals(channel,:) = AddWhiteNoise(auxSignals(channel,:),experimentOptions.dataOptions.snrValues(sSNR));
                end
            end
            signals = cat(1,signals,auxSignals);
        end 
    end
    
%     %%% Save'em
%     save(strcat('/local_scratch/alamedap/dataGeorge/UpToSubsample_',num2str(sSignal),'_',num2str(sPosition),'_',num2str(sSNR),'_',num2str(sT60),'.mat'),'signals');
end