function [solverOutput, methodOptions] = NGTDE(PCCC, microphones, samplingPeriod, x0, methodOptions, energies)

%Geometric-constrained time difference estimation
%
% USAGE: [solverOutput, methodOptions] = NGTDE(PCCC, microphones, samplingPeriod, x0, methodOptions[, energies])
%
% PARAMETERS:
%   XCPC ~ the interpolation coefficients of the cross-correlation function
%   microphonePositions ~ positions of the microphones
%   samplingPeriod ~ sampling period of the signal
%   x0 ~ initialization positions for the local optimizer
%   methodOptions ~ method options
%
% RESULT:
%   solverOutput ~ the solution of the solver
%   methodOptions ~ the method options eventually modified
%
% DESCRIPTION:
%     Estimates the time delay between the signals. XCPC represents the
%     coefficients of the spline interpolation of the cross-correlation
%     function. The optimization problem minimizes the multichannel
%     criterion. A dual-interior-point algorithm is used.
%
% see also MultichannelTDECriterion, ipsolver_parallel

% Copyright 2012, Xavier Alameda-Pineda
% INRIA Grenoble Rhône-Alpes
% E-mail: xavi.alameda@gmail.com
% 
% This is part of the gtde program.
% 
% gtde is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
    %%% Input check
    if nargin < 5
        error('Usage [solverOutput, methodOptions] = NGTDE(PCCC, microphones, samplingPeriod, x0, methodOptions[, energies])');
    end
    if size(PCCC,1) ~= size(microphones,1)
        error('There should be as many signals as microphones.');
    end
    if size(microphones,2) ~= 3 && size(microphones,2) ~= 2
        error('The system works in 2D or in 3D.');
    end
    if samplingPeriod <= 0
        error('The sampling period should be positive.');
    end
        
    % Declare the objective function
    if nargin < 6
        objFunction = @(x) MultichannelTDECriterion(x,PCCC,microphones,samplingPeriod);
    else
        objFunction = @(x) MultichannelTDECriterion(x,PCCC,microphones,samplingPeriod,energies);
    end
    consFunction = @(x,z) DummyConstraint(x,z);

    % Run the parallel local optimization
    [solverOutput, methodOptions] = IPSolverParallel(x0,objFunction,consFunction,methodOptions);
    
    % Revers the constraint value
    solverOutput.c = -solverOutput.c;
    if isfield(solverOutput,'cIT');
        solverOutput.cIT = -solverOutput.cIT;
    end
end


