function [XCPC, Energies] = Signals2XCPC(signals,MICS,samplingFrequency)

%Geometric-constrained time difference estimation
%
% USAGE:  PCCC = Signals2PCCC(signals,MICS,samplingFrequency)
%
% PARAMETERS:
%     sigPCCC ~ polynomial cross correlation of the signals' coefficients
%     microphones ~ positions of the microphones
%     samplingPeriod ~ sampling period of the signal
%     x0 ~ initialization points of the local minimizer
%
% RESULT:
%     The time delay estimates.
%
% DESCRIPTION:
%     Estimates the time difference of signals (or their Polynomial 
%     Coefficients Cross-Correlation) acquired by microphones 
%     sampled at samplingPeriod. The method is based on a geometric-
%     constrained maximization of the continuous estimation of the cross-
%     correlation functions. 'signals' is a M-by-N matrix with one row per 
%     acquired signal. 'microphoned' is a M-by-3 matrix with one row per 
%     microphone's position. TDE is a M(M-1)/2 vector with the TDEs. 
% 
%   see also PolynomialInterpolationCoefficients, ipsolver_parallel, gTDELogCriterion


% Copyright 2012, Xavier Alameda-Pineda
% INRIA Grenoble Rhône-Alpes
% E-mail: xavi.alameda@gmail.com
% 
% This is part of the gtde program.
% 
% gtde is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

    % Check input
    if nargin < 3
        error('Usage:  [XCPC Energies] = Signals2XCPC(signals,MICS,samplingFrequency).');
    end
    
    % Number of microphones
    NMics = size(signals,1);
    
    % Compute maxlag
    maxLag = TDEmax(MICS);
    maxLag = ceil(1.1*samplingFrequency*max(maxLag));
    
    % Discrete cross-correlation values and interpolation coefficients
    XCPC = cell(NMics);
    Energies = zeros(1,NMics);
    % Loop on the microphones
    for mic1 = 1:NMics,
        for mic2 = 1:NMics,
            % Compute discrete cross-correlation
            xcValues = DiscreteCrossCorrelation(signals(mic1,:),signals(mic2,:),maxLag);
            % Interpolate
            XCPC{mic1,mic2} = PolynomialInterpolationCoefficients(xcValues,1/samplingFrequency)';
            % Look for the energy if mic1 == mic2
            if mic1 == mic2
                % Aux variables
                a0 = XCPC{mic1,mic1}(1,:);
                a1 = XCPC{mic1,mic1}(2,:);
                a2 = XCPC{mic1,mic1}(3,:);
                a3 = XCPC{mic1,mic1}(4,:);
                % Potential energies: first the limits of the interpolation
                % itervals and then possible extrema inside the interval
                PotentialEnergies = a0;
                % The extrema inside the interval are those which cancel
                % the first derivative. This is a second degree equation,
                % with the following discriminant.
                Discriminant = a2.^2 - 3*a1.*a3;
                ind = Discriminant >= 0;
                Discriminant = Discriminant(ind);
                % Whenever the discriminant is non-negative, we manage to
                % recover points the vanish the first derivative. If those
                % points are inside the interval, we use them to compute
                % possible maxima/minima.
                a0 = a0(ind);
                a1 = a1(ind);
                a2 = a2(ind);
                a3 = a3(ind);
                % Positive solution of the equation
                tValues = (-a2 + sqrt(Discriminant))./(3*a3);
                ind = 0 <= tValues & tValues <= 1/samplingFrequency;
                PotentialEnergies = cat(2,PotentialEnergies,...
                    a0(ind) + a1(ind).*tValues(ind) + a2(ind).*tValues(ind).^2 + a3(ind).*tValues(ind).^3);
                % Negative solution of the equation
                tValues = (-a2 + sqrt(Discriminant))./(3*a3);
                ind = 0 <= tValues & tValues <= 1/samplingFrequency;
                PotentialEnergies = cat(2,PotentialEnergies,...
                    a0(ind) + a1(ind).*tValues(ind) + a2(ind).*tValues(ind).^2 + a3(ind).*tValues(ind).^3);
                % Energy
                Energies(mic1) = max(abs(PotentialEnergies));
            end
        end
    end
    
    %%% Correction factor for the energies, it turns out that picking the
    % maximum of the interpolated auto-correlation does not guarantee
    % that the normalized cross-correlation will be between -1 and 1. So
    % this factor is used to heuristically guarantee that.
    Energies = Energies * 1.02;
end