function [TDEs] = TDEGeometricDirect(sourcePosition, microphonePositions, C)

% TDEGeometricDirect geometric model for TDE
% 
%   TDEGeometricDirect(sourcePosition, microphonePositions) returns the TDE 
%   values that a source on sourcePosition generates with the microphones 
%   placed at microphonePositions. 
%   
%   TDEGeometricDirect(sourcePosition, microphonePositions) returns the 
%   same as TDEGeometricDirect(sourcePosition, microphonePositions, C) with 
%   the specified sound propagation speed. The default is C = 343.2
%   
%   see also EuclideanDistance
%

% Copyright 2012, Xavier Alameda-Pineda
% INRIA Grenoble Rhône-Alpes
% E-mail: xavi.alameda@gmail.com
% 
% This is part of the gtde program.
% 
% gtde is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

    %%% Check Input
    % If less than two arguments
    if(nargin < 2)
        error('Usage: [TDEs Jacobians] = TDEGeometricDirect(X, MICS[, C])');
    end

    % Default argument for sound propagation speed
    if(nargin < 3)
        C = 343.2;
    end

    % Check the dimension
    if size(sourcePosition,2)~=size(microphonePositions,2)
        error('X and MICS should have the same number of columns.');
    end

    % Number of microphones and points
    NMics = size(microphonePositions,1);
    NPoints = size(sourcePosition,1);

    % Number of microphones
    if NMics < 2
        error('At least two microphones are needed to compute TDEs.');
    end

    % Number of points
    if NPoints < 1
        error('At least one point is needed to compute TDEs.');
    end

    % Useful parameters
    Dimension = size(sourcePosition,2);% = size(MICS,2)
    NTDEs = NMics*(NMics-1)/2;

    %%% Compute the ITD values

    % Compute the distance to each microphone
    DMICS = zeros(NPoints,NMics);
    for mic=1:NMics,
        DMICS(:,mic) = EuclideanDistance(sourcePosition,microphonePositions(mic,:));
    end

    TDEs = zeros(NPoints,NTDEs);

    % Compute the time difference of each microphone pair
    for mic1 = 1:NMics,
        for mic2 = mic1+1:NMics,
            % Compute the index corresponding to the mic1,mic2
            si = TDEis(mic1,mic2,NMics);
            TDEs(:,si) = (DMICS(:,mic2) - DMICS(:,mic1))/C;
        end
    end

%     %%% Compute the jacobian if desired
% 
%     % If desired, compute the Jacobian at each point
%     if nargout > 1
%         % Initialize the Jacobians varaible
%         Jacobians = zeros(NPoints,NTDEs,Dimension);
% 
%         % Compute the directions from the sources to the microphones
%         DIRMICS = zeros(NPoints,NMics,Dimension);
%         for mic = 1:NMics
%             DIRMICS(:,mic,:) = bsxfun(@minus,sourcePosition,microphonePositions(mic,:))./repmat(DMICS(:,mic),1,Dimension);
%         end
% 
%         % Compute the Jacobians
%         for mic1 = 1:NMics,
%             for mic2 = mic1+1:NMics,
%                 % Compute the index corresponding to the mic1,mic2
%                 si = TDEis(mic1,mic2,NMics);
%                 Jacobians(:,si,:) = (DIRMICS(:,mic2,:) - DIRMICS(:,mic1,:))/C;
%             end
%         end
%     end
    
end % function [TDEs Jacobians] = TDEGeometricDirect(X, MICS, C)