function Source = TDEGeometricInverse(MICS,TDEs,C,mode)

% TDEGeometricInverse estimates the sound source position
%   The sound source is estimated from the TDEs. The assumption is that
%   all the observed TDE values correspond to the same sound source. 
% 
%   TDEGeometricInverse(MICS, TDEs) returns an estimate of the sound source 
%   position given the collected TDEs from the microphones placed at MICS. 
%   The size of TDEs must be NMics-1. In the first case the TDEs are the 
%   difference of time of arrival with respect to the first microphone.
%     
%   TDEGeometricInverse(MICS, TDEs, C) returns an estimate of the sound 
%   source position specifying the sound propagation speed. 
%   The default is C = 343.2 [m/s]
%
%   TDEGeometricInverse(MICS, TDEs, C, mode) if mode is set to 'all', the
%   function will return at most two positions for the sound source. The
%   two solutions of the second degree equation. Set the mode to 'one' will
%   act as previously described. Default is 'one'.
%   
%   References:
%     [1] Chan, Y.T. and Ho, K.C., A simple and efficient estimator for 
%     hyperbolic location, Signal Processing, IEEE Transactions on, 1994.

% Copyright 2012, Xavier Alameda-Pineda
% INRIA Grenoble Rhône-Alpes
% E-mail: xavi.alameda@gmail.com
% 
% This is part of the gtde program.
% 
% gtde is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.

    %%% Input check
    if nargin < 2
        error('Usage: Source = TDEGeometricInverse(MICS, TDEs [, C])');
    end

    % Default value for the sound speed
    if nargin < 3
        C = 343.2;
    end
    
    % Default mode
    if nargin < 4
        mode = 'one';
    else
        if ~strcmp(mode,'all') && ~strcomp(mode,'one'),
            error('If desired, mode should be set eigher to "all" or to "one".');
        end
    end

    % Number of microphones
    NMics = size(MICS,1);
    % Dimension
    Dimension = size(MICS,2);
    if ~(NMics==4 && Dimension==3) && ~(NMics==3 && Dimension==2)
        error('This function is ready to work in 3D with four microphones or in 2D with three microphones.');
    end

    % Check the number of TDEs
    if numel(TDEs) ~= NMics-1
        error('The number of TDEs should be NMics-1.');
    end

    %%% Compute the sound source potision
    % 1. Compute useful parameters

    % Module of the receivers' position
    K = sum( MICS.^2, 2) - sum(MICS(1,:).^2);
    K(1) = [];
    K1 = sum(MICS(1,:).^2);
    % DDOA ~ Distance Difference of Arrival
    R = C*TDEs';
    % Receivers' position difference
    % Here is why it is important to have 4 mics and to be in 3D
    % since this matrix will be inversed during the method
    M = zeros(NMics-1,Dimension);
    for mic=2:NMics,
        M(mic-1,:) = MICS(mic,:)-MICS(1,:);
    end
    M = 2*M;

    % 2. Start the computation
    Minv = pinv(M);

    % A vector
    A = -2*R;
    A = Minv*A;

    % B vector
    B = K - R.^2;
    B = Minv*B;
    BM1 = B - MICS(1,:)';

    % Coefficients of the 2nd order equation
    c2 = norm(A,2).^2-1;
    c1 = 2*sum(A.*BM1);
    c0 = norm(BM1,2).^2;

    % Check the discriminant and the higher order coefficient
    % of the second order equation.
    Source = [];
    if c1^2 - 4*c2*c0 >= 0 && c2 ~= 0
        % Solve it
        r1p = (-c1 + sqrt(c1.^2 - 4*c2*c0))/(2*c2);
        r1n = (-c1 - sqrt(c1.^2 - 4*c2*c0))/(2*c2);

        % 3. Compute the sources
        Source = [A*r1p+B, A*r1n+B]'; 
        
        % If desired, choose the right one
        if strcmp(mode,'one')
%             fprintf('%0.8g  %0.8g\n',r1p,r1n);
                AuxTDEs = TDEGeometricDirect(Source(1,:),MICS,C);
            if sum( (AuxTDEs(1:NMics-1)-TDEs).^2 ) > 1e-8
%                 plot3(Source(2,1),Source(2,2),Source(2,3),'go');
%                 plot3(Source(1,1),Source(1,2),Source(1,3),'rx');
                Source = Source(2,:);
%                 if r1n < 0 || r1p > 0
%                     fprintf('%.2g\n',r1n);
%                     fprintf('%.4g ',Source);
%                     fprintf('\n');
%                 end
            else
%                 plot3(Source(2,1),Source(2,2),Source(2,3),'rx');
%                 plot3(Source(1,1),Source(1,2),Source(1,3),'go');
                Source = Source(1,:);
%                 if r1p < 0 || r1n > 0
%                     fprintf('%.2g\n',r1p);
%                     fprintf('%.4g ',Source);
%                     fprintf('\n');
%                 end
            end
        end
%     else
%         disp('what?');
    end

end % function Source = TDEGeometricInverse(MICS,TDEs,C)

